@@ -1,19 +1,19 @@
 #' Pad 3D Array
 #'
 #' Pad a 3D array by a certain amount in each direction, along each dimension.
-#'  This operation is like the opposite of cropping. 
+#'  This operation is like the opposite of cropping.
 #'
-#' @param x A 3D array, e.g. 
+#' @param x A 3D array, e.g.
 #'  \code{unvec_vol(xifti$data$subcort, xifti$meta$subcort$mask)}.
-#' @param padding A \eqn{3 \times 2} matrix indicating the number of 
-#'  slices to add at the beginning (first column) and end (second column) of 
+#' @param padding A \eqn{3 \times 2} matrix indicating the number of
+#'  slices to add at the beginning (first column) and end (second column) of
 #'  each of dimension, e.g. \code{xifti$meta$subcort$mask_padding}.
 #' @param fill Value to pad with. Default: \code{NA}.
 #'
 #' @return The padded array
 #'
 #' @export
-#' 
+#'
 #' @examples
 #' x <- array(seq(24), dim=c(2,3,4))
 #' y <- pad_vol(x, array(1, dim=c(3,2)), 0)
@@ -45,7 +45,7 @@ pad_vol <- function(x, padding, fill=NA){
 }
 
 #' @rdname pad_vol
-#' 
+#'
 uncrop_vol <- function(x, padding, fill=NA){
   pad_vol(x, padding, fill)
 }
@@ -53,24 +53,24 @@ uncrop_vol <- function(x, padding, fill=NA){
 #' Sum of each voxel's neighbors
 #'
 #' For each voxel in a 3D logical or numeric array, sum the values of the six
-#'  neighboring voxels. 
-#' 
+#'  neighboring voxels.
+#'
 #' Diagonal voxels are not considered adjacent, i.e. the voxel at (0,0,0) is not
-#'  adjacent to the voxels at (1,1,0) or (1,1,1), although it is adjacent to 
+#'  adjacent to the voxels at (1,1,0) or (1,1,1), although it is adjacent to
 #'  (1,0,0).
 #'
 #' @param arr The 3D array.
 #' @param pad In order to compute the sum, the array is temporarily padded along
 #'  each edge with the value of \code{pad}. \code{0} (default) will mean that
 #'  edge voxels reflect the sum of 3-5 neighbors whereas non-edge voxels reflect
-#'  the sum of 6 neighbors. An alternative is to use a value of \code{NA} so 
+#'  the sum of 6 neighbors. An alternative is to use a value of \code{NA} so
 #'  that edge voxels are \code{NA}-valued because they did not have a complete
 #'  set of six neighbors. Perhaps another option is to use \code{mean(arr)}.
-#' 
+#'
 #' @return An array with the same dimensions as \code{arr}. Each voxel value
 #'  will be the sum across the immediate neighbors. If \code{arr} was a logical
 #'  array, this value will be between 0 and 6.
-#' 
+#'
 #' @export
 sum_neighbors_vol <- function(arr, pad=0){
   # Argument checks.
@@ -94,18 +94,18 @@ sum_neighbors_vol <- function(arr, pad=0){
 #'
 #' Erode a volumetric mask by a certain number of voxel layers. For each layer,
 #'  any in-mask voxel adjacent to at least one out-of-mask voxel is removed
-#'  from the mask. 
+#'  from the mask.
 #'
 #' Diagonal voxels are not considered adjacent, i.e. the voxel at (0,0,0) is not
-#'  adjacent to the voxels at (1,1,0) or (1,1,1), although it is adjacent to 
+#'  adjacent to the voxels at (1,1,0) or (1,1,1), although it is adjacent to
 #'  (1,0,0).
 #'
 #' @param vol The 3D array to erode. The mask to erode is defined by all values
 #'  not in \code{out_of_mask_val}.
-#' @param n_erosion The number of layers to erode the mask by. Default: 
+#' @param n_erosion The number of layers to erode the mask by. Default:
 #'  \code{1}.
 #' @param out_of_mask_val A voxel is not included in the mask if and only if its
-#'  value is in this vector. The first value of this vector will be used to 
+#'  value is in this vector. The first value of this vector will be used to
 #'  replace eroded voxels. Default: \code{NA}. If \code{vol} is simply a logical
 #'  array with \code{TRUE} values for in-mask voxels, use
 #'  \code{out_of_mask_val=FALSE}.
@@ -130,25 +130,68 @@ erode_mask_vol <- function(vol, n_erosion=1, out_of_mask_val=NA){
   vol
 }
 
+#' Dilate 3D mask
+#'
+#' Dilate a volumetric mask by a certain number of voxel layers. For each layer,
+#'  any out-of-mask voxel adjacent to at least one in-mask voxel is added to the
+#'  mask.
+#'
+#' Diagonal voxels are not considered adjacent, i.e. the voxel at (0,0,0) is not
+#'  adjacent to the voxels at (1,1,0) or (1,1,1), although it is adjacent to
+#'  (1,0,0).
+#'
+#' @param vol The 3D array to dilate. The mask to dilate is defined by all
+#'  values not in \code{out_of_mask_val}.
+#' @param n_dilate The number of layers to dilate the mask by. Default:
+#'  \code{1}.
+#' @param out_of_mask_val A voxel is not included in the mask if and only if its
+#'  value is in this vector. Default: \code{NA}. If \code{vol} is simply a
+#'  logical array with \code{TRUE} values for in-mask voxels, use
+#'  \code{out_of_mask_val=FALSE}.
+#' @param new_val Value for voxels newly added to the mask. Default: \code{1}.
+#'  If \code{vol} is simply a logical array with \code{TRUE} values for
+#'  in-mask voxels, use \code{new_val=1}.
+#'
+#' @return The dilated \code{vol}. It is the same as \code{vol}, but dilated
+#'  voxels are replaced with \code{new_val}.
+#'
+#' @export
+dilate_mask_vol <- function(vol, n_dilate=1, out_of_mask_val=NA, new_val=1){
+  # Argument checks.
+  stopifnot(length(dim(vol)) == 3)
+  stopifnot(is_integer(n_dilate, nneg=TRUE))
+  stopifnot(is_1(new_val, "numeric") | is_1(new_val, "logical"))
+  if (n_dilate == 0) { return(vol) }
+
+  # Dilate.
+  mask <- !array(vol %in% out_of_mask_val, dim=dim(vol))
+  for (ii in seq(n_dilate)) {
+    to_dilate <- (!mask) & (sum_neighbors_vol(mask, pad=0) > 0)
+    mask[to_dilate] <- TRUE
+    vol[to_dilate] <- new_val
+  }
+  vol
+}
+
 #' Convert coordinate list to 3D array
-#' 
+#'
 #' Converts a sparse coordinate list to its non-sparse volumetric representation.
-#' 
-#' @param coords The sparse coordinate list. Should be a \code{"data.frame"} or 
-#'  matrix with voxels along the rows and three or four columns. The first three 
+#'
+#' @param coords The sparse coordinate list. Should be a \code{"data.frame"} or
+#'  matrix with voxels along the rows and three or four columns. The first three
 #'  columns should be integers indicating the spatial coordinates of the voxel.
 #'  If the fourth column is present, it will be the value used for that voxel.
 #'  If it is absent, the value will be \code{TRUE} or \code{1} if \code{fill}
-#'  is not one of those values, and \code{FALSE} or \code{0} if \code{fill} is. 
-#'  The data type will be the same as that of \code{fill}. The fourth column 
+#'  is not one of those values, and \code{FALSE} or \code{0} if \code{fill} is.
+#'  The data type will be the same as that of \code{fill}. The fourth column
 #'  must be logical or numeric.
-#' @param fill Logical or numeric fill value for the volume. Default: 
+#' @param fill Logical or numeric fill value for the volume. Default:
 #'  \code{FALSE}.
-#' 
+#'
 #' @return The volumetric data
 #'
 #' @export
-#' 
+#'
 coordlist_to_vol <- function(coords, fill=FALSE){
   # Check arguments.
   stopifnot(length(fill) == 1)
@@ -168,8 +211,8 @@ coordlist_to_vol <- function(coords, fill=FALSE){
     )
     coords <- cbind(coords, val)
   } else {
-    if (any(coords[,4] == fill, na.rm=TRUE)) { 
-      warning("The fill value occurs in the data.") 
+    if (any(coords[,4] == fill, na.rm=TRUE)) {
+      warning("The fill value occurs in the data.")
     }
   }
 
@@ -179,16 +222,16 @@ coordlist_to_vol <- function(coords, fill=FALSE){
 }
 
 #' Crop a 3D array
-#' 
+#'
 #' Remove empty (zero-valued) edge slices from a 3D array.
 #'
 #' @param x The numeric 3D array to crop.
 #' @return A list of length two: \code{"data"}, the cropped array, and
-#'  \code{"padding"}, the number of slices removed from each edge of each 
+#'  \code{"padding"}, the number of slices removed from each edge of each
 #'  dimension.
-#' 
+#'
 #' @export
-#' 
+#'
 crop_vol <- function(x) {
   # Argument checks.
   d <- length(dim(x))
@@ -205,19 +248,19 @@ crop_vol <- function(x) {
     first_slice <- min(which(!empty_slice[[ii]]))
     last_slice <- max(which(!empty_slice[[ii]]))
     padding[ii,1] <- ifelse(
-      first_slice != 1, 
-      first_slice - 1, 
+      first_slice != 1,
+      first_slice - 1,
       0
     )
     padding[ii,2] <- ifelse(
-      last_slice != length(empty_slice[[ii]]), 
-      length(empty_slice[[ii]]) - last_slice, 
+      last_slice != length(empty_slice[[ii]]),
+      length(empty_slice[[ii]]) - last_slice,
       0
     )
   }
 
   # Crop.
-  # This can easily be generalized to support arrays with number of dimensions 
+  # This can easily be generalized to support arrays with number of dimensions
   #   other than 3. But for now, it only works for 3D arrays.
   x <- x[!empty_slice[[1]], !empty_slice[[2]], !empty_slice[[3]]]
 
@@ -225,19 +268,19 @@ crop_vol <- function(x) {
 }
 
 #' Get coordinates of each voxel in a mask
-#' 
+#'
 #' Made for obtaining voxel locations in 3D space from the subcortical metadata
-#'  of CIFTI data: the volumetric mask, the transformation matrix and the 
+#'  of CIFTI data: the volumetric mask, the transformation matrix and the
 #'  spatial units.
-#' 
+#'
 #' @param mask 3D logical mask
 #' @param trans_mat Transformation matrix from array indices to spatial
 #'  coordinates.
 #' @param trans_units Units for the spatial coordinates (optional).
 #' @return A list: \code{coords} and \code{trans_units}.
-#' 
+#'
 #' @export
-#' 
+#'
 vox_locations <- function(mask, trans_mat, trans_units=NULL){
   # Argument checks.
   stopifnot(dim(mask) == 3)
@@ -251,13 +294,13 @@ vox_locations <- function(mask, trans_mat, trans_units=NULL){
 }
 
 #' Convert vectorized data back to volume
-#' 
+#'
 #' Un-applies a mask to vectorized data to yield its volumetric representation.
 #'  The mask and data should have compatible dimensions: the number of rows in
 #'  \code{dat} should equal the number of locations within the \code{mask}.
-#' 
-#' @param dat Data matrix with locations along the rows and measurements along 
-#'  the columns. If only one set of measurements were made, this may be a 
+#'
+#' @param dat Data matrix with locations along the rows and measurements along
+#'  the columns. If only one set of measurements were made, this may be a
 #'  vector.
 #' @param mask Volumetric binary mask. \code{TRUE} indicates voxels inside the
 #'  mask.
@@ -266,7 +309,7 @@ vox_locations <- function(mask, trans_mat, trans_units=NULL){
 #' @return The 3D or 4D unflattened volume array
 #'
 #' @export
-#' 
+#'
 unvec_vol <- function(dat, mask, fill=NA) {
 
   # Check that dat is a vector or matrix.
