@@ -20,8 +20,8 @@ test_that("Miscellaneous functions are working", {
   plot_FC(mat, zlim=c(-.8, .8), diag_val=1, title="ABC", cleg_ticks_by=.2, lines="all", lines_col="white")
   plot_FC_gg(mat)
   plot_FC_gg(mat, title="ABC", legTitle="DEF", lim=c(-.8, .8), diagVal=0)
-  plot_FC_gg(mat, y_labs=as.character(seq(10)))
+  plot_FC_gg(mat, labs=as.character(seq(10)))
   plot_FC_gg(mat, group_divs=c(1,2,5,8,10))
-  plot_FC_gg(mat, group_divs=c(2,10), uppertri_means=FALSE, divColor="white", y_labs=as.character(seq(10)))
-  plot_FC_gg(mat, group_divs=c(1,2,5,8,10), y_labs=c("a", "b", "c", "d", "e"))
+  plot_FC_gg(mat, group_divs=c(2,10), uppertri_means=FALSE, divColor="white", labs=as.character(seq(10)))
+  plot_FC_gg(mat, group_divs=c(1,2,5,8,10), labs=c("a", "b", "c", "d", "e"))
 })
